"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseMidi = void 0;
var debug_1 = require("./debug");
var FileParser_1 = require("./FileParser");
var parseMidi = function (buffer) {
    var uint8Data = new Uint8Array(buffer);
    var file = new FileParser_1.FileParser(new DataView(uint8Data.buffer, uint8Data.byteOffset, uint8Data.byteLength));
    var headerValidation = file.readInt(4);
    if (headerValidation !== 0x4d546864) {
        debug_1.log("Header validation failed (not MIDI standard or file corrupt), code: 0x" + headerValidation.toString(16));
        return null;
    }
    file.movePointer(4);
    var parsedMidi = {
        formatType: file.readInt(2),
        tracks: file.readInt(2),
        track: [],
        timeDivision: [],
    };
    var timeDivisionByte1 = file.readInt(1);
    var timeDivisionByte2 = file.readInt(1);
    if (timeDivisionByte1 >= 128) {
        parsedMidi.timeDivision = [timeDivisionByte1 - 128, timeDivisionByte2];
    }
    else {
        parsedMidi.timeDivision = timeDivisionByte1 * 256 + timeDivisionByte2;
    }
    for (var t = 1; t <= parsedMidi.tracks; t += 1) {
        var track = {
            events: [],
            text: [],
            copyright: [],
            trackName: [],
            instrumentName: [],
            programName: [],
            lyric: [],
            marker: [],
            cuePoint: [],
            tempo: [],
            smpteOffset: [],
            timeSignature: [],
            keySignature: [],
            notes: [],
            programChange: [],
        };
        parsedMidi.track.push(track);
        var headerValidation_1 = file.readInt(4);
        if (headerValidation_1 === -1) {
            break;
        }
        if (headerValidation_1 !== 0x4d54726b) {
            debug_1.log("Header validation failed (not MIDI standard or file corrupt), code: 0x" + headerValidation_1.toString(16));
            return null;
        }
        file.movePointer(4);
        var endOfTrack = false;
        var lastStatusByte = -1;
        while (!endOfTrack) {
            var time = file.readIntVLV();
            var statusByte = file.readInt(1);
            if (statusByte === -1) {
                break;
            }
            else if (statusByte >= 128) {
                lastStatusByte = statusByte;
            }
            else {
                statusByte = lastStatusByte;
                file.movePointer(-1);
            }
            if (statusByte === 0xff) {
                var type = file.readInt(1);
                if (type === 0x2f) {
                    endOfTrack = true;
                    file.movePointer(1);
                }
                else {
                    getMetaData(type, time, track, file);
                }
            }
            else {
                var type = statusByte >> 4;
                var channel = statusByte & 0xf;
                if (!getRegularData(type, channel, time, track, file)) {
                    debug_1.log("Unknown event 0x" + type.toString(16) + " detected! Reading cancelled!");
                    return null;
                }
            }
        }
    }
    return parsedMidi;
};
exports.parseMidi = parseMidi;
function getRegularData(type, channel, time, track, file) {
    switch (type) {
        case 0xf: {
            var event_length = file.readIntVLV();
            file.movePointer(event_length);
            debug_1.log("Unimplemented 0xF exclusive events! Skipped.");
            break;
        }
        case 0x8:
            track.notes.push({
                time: time,
                on: false,
                note: file.readInt(1),
                velocity: file.readInt(1),
                channel: channel,
            });
            break;
        case 0x9:
            track.notes.push({
                time: time,
                on: false,
                note: file.readInt(1),
                velocity: file.readInt(1),
                channel: channel,
            });
            break;
        case 0xc:
            track.programChange.push({
                time: time,
                program: file.readInt(1),
                channel: channel,
            });
            break;
        case 0xd:
            track.events.push({
                channel: channel,
                time: time,
                type: type,
                data: [file.readInt(1)],
            });
            break;
        case 0xa:
        case 0xb:
        case 0xe:
            track.events.push({
                channel: channel,
                time: time,
                type: type,
                data: [file.readInt(1), file.readInt(1)],
            });
            break;
        default:
            return false;
    }
    return true;
}
function getMetaData(type, time, track, file) {
    var metaEventLength = file.readIntVLV();
    switch (type) {
        case 0x01:
            track.text.push({ time: time, text: file.readStr(metaEventLength) });
            break;
        case 0x02:
            track.copyright.push({
                time: time,
                text: file.readStr(metaEventLength),
            });
            break;
        case 0x03:
            track.trackName.push({
                time: time,
                text: file.readStr(metaEventLength),
            });
            break;
        case 0x04:
            track.instrumentName.push({
                time: time,
                text: file.readStr(metaEventLength),
            });
            break;
        case 0x05:
            track.lyric.push({
                time: time,
                text: file.readStr(metaEventLength),
            });
            break;
        case 0x06:
            track.marker.push({
                time: time,
                text: file.readStr(metaEventLength),
            });
            break;
        case 0x07:
            track.cuePoint.push({
                time: time,
                text: file.readStr(metaEventLength),
            });
            break;
        case 0x08:
            track.programName.push({
                time: time,
                text: file.readStr(metaEventLength),
            });
            break;
        case 0x51:
            track.tempo.push({
                tempo: file.readInt(metaEventLength),
                time: time,
            });
            break;
        case 0x54:
            track.smpteOffset.push({
                time: time,
                hr: file.readInt(1),
                mn: file.readInt(1),
                se: file.readInt(1),
                fr: file.readInt(1),
                ff: file.readInt(1),
            });
            break;
        case 0x58:
            track.timeSignature.push({
                time: time,
                nn: file.readInt(1),
                dd: file.readInt(1),
                cc: file.readInt(1),
                bb: file.readInt(1),
            });
            break;
        case 0x59:
            track.keySignature.push({
                time: time,
                sf: file.readInt(1),
                mi: file.readInt(1),
            });
            break;
        default:
            debug_1.log("Unimplemented meta event, 0x" + type.toString(16) + " detected! Skipped.");
            file.movePointer(metaEventLength);
    }
}
//# sourceMappingURL=midiParser.js.map